from django.conf import settings
from django.contrib import admin
from django.urls import include, path

urlpatterns = [
    path(r"", admin.site.urls),
]

if settings.DEBUG:
    import debug_toolbar

    urlpatterns += [path("__debug__/", include(debug_toolbar.urls))]



from crispy_forms.helper import FormHelper
from crispy_forms.layout import BaseInput, ButtonHolder, Field, Fieldset, Layout, Row
from django import forms
from django.contrib.auth import forms as auth_forms
from django.contrib.auth import get_user_model, password_validation
from django.contrib.auth.forms import PasswordChangeForm
from django.core.mail import EmailMultiAlternatives
from django.template import loader
from django_countries import countries
from django_q.tasks import async_task

from tuhls.core.forms import TailwindFormMixin, TailwindModelForm


class Button(BaseInput):
    input_type = "submit"

    def __init__(self, *args, **kwargs):
        self.field_classes = (
            "inline-flex justify-center py-3 px-6 border border-transparent shadow-sm text-base font-medium"
            + "rounded-md text-white bg-pc-600 hover:bg-pc-700 focus:outline-none focus:ring-2"
            + "focus:ring-offset-2 focus:ring-pc-500"
        )
        super().__init__(*args, **kwargs)


class EmptyChoiceField(forms.ChoiceField):
    def __init__(
        self,
        choices=(),
        empty_label=None,
        required=True,
        widget=None,
        label=None,
        initial=None,
        help_text=None,
        *args,
        **kwargs
    ):

        # prepend an empty label if it exists (and field is not required!)
        if not required and empty_label is not None:
            choices = tuple([("", empty_label)] + list(choices))

        super().__init__(
            choices=choices,
            required=required,
            widget=widget,
            label=label,
            initial=initial,
            help_text=help_text,
            *args,
            **kwargs
        )


class ButtonFullWidth(BaseInput):
    input_type = "submit"

    def __init__(self, *args, **kwargs):
        self.field_classes = (
            "w-full flex justify-center py-2 px-4 border border-transparent rounded-md shadow-sm text-sm"
            + "font-medium text-white bg-pc-600 hover:bg-pc-700 focus:outline-none focus:ring-2"
            + "focus:ring-offset-2 focus:ring-pc-500"
        )
        super().__init__(*args, **kwargs)


class TailwindForm:
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.helper = FormHelper()
        self.helper.form_class = "grid grid-cols-1 gap-y-6"
        self.helper.label_class = "block text-sm font-medium text-gray-700 mb-1"


class PasswordResetForm(auth_forms.PasswordResetForm):
    def send_mail(
        self,
        subject_template_name,
        email_template_name,
        context,
        from_email,
        to_email,
        html_email_template_name=None,
    ):
        subject = loader.render_to_string(subject_template_name, context)
        # Email subject *must not* contain newlines
        subject = "".join(subject.splitlines())
        body = loader.render_to_string(email_template_name, context)

        email_message = EmailMultiAlternatives(subject, body, from_email, [to_email])
        if html_email_template_name is not None:
            html_email = loader.render_to_string(html_email_template_name, context)
            email_message.attach_alternative(html_email, "text/html")

        async_task(email_message.send)


class SetPasswordForm(TailwindForm):
    new_password = forms.CharField(
        label="New password",
        widget=forms.PasswordInput(attrs={"autocomplete": "new-password"}),
        strip=False,
    )

    def __init__(self, user, *args, **kwargs):
        self.user = user
        super().__init__(*args, **kwargs)
        self.helper.layout = Layout(
            "new_password",
            ButtonHolder(ButtonFullWidth("submit", "Change my Password")),
        )

    def clean_new_password(self):
        password = self.cleaned_data.get("new_password")
        password_validation.validate_password(password, self.user)
        return password

    def save(self, commit=True):
        password = self.cleaned_data["new_password"]
        self.user.set_password(password)
        if commit:
            self.user.save()
        return self.user


class SignupForm(TailwindModelForm):
    email = forms.EmailField(required=True)
    password = forms.CharField(
        required=True, widget=forms.PasswordInput(render_value=True)
    )

    news = forms.BooleanField(
        label="Receive important maintenance and service notifications",
        initial=True,
        required=False,
    )

    tos = forms.BooleanField(
        label=(
            "I have read and agree to the Terms & Conditions, Privacy Policy and Service Agreement "
            + "of this website and service."
        ),
        error_messages={
            "required": "You have to accept the Terms & Conditions, Privacy Policy and Service agreement."
        },
        initial=False,
        required=True,
    )

    class Meta:
        model = get_user_model()
        fields = (
            "email",
            "tos",
            "news",
        )

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

        self.helper.layout = Layout(
            Fieldset(
                "Account Details",
                "email",
                "password",
            ),
            Fieldset(
                "Legal",
                Field("news", template="crispy/checkboxfield.html"),
                Field("tos", template="crispy/checkboxfield.html"),
                css_class="text-justify block",
            ),
            ButtonHolder(ButtonFullWidth("submit", "Sign Up", css_class="mt-4")),
        )

    def save(self, commit=True) -> get_user_model():
        user = super().save(commit=False)
        user.set_password(self.cleaned_data["password"])
        if commit:
            user.save()
        return user


class AccountForm(TailwindModelForm):
    first_name = forms.CharField(required=False)
    last_name = forms.CharField(required=False)
    address = forms.CharField(required=False)
    postal_code = forms.CharField(required=False)
    country = forms.ChoiceField(
        required=False, widget=forms.Select(), choices=countries
    )
    state = forms.CharField(required=False)
    city = forms.CharField(required=False)

    company_name = forms.CharField(required=False)
    company_website = forms.CharField(required=False)
    company_vat = forms.CharField(required=False)

    news = forms.BooleanField(
        label="Receive important maintenance and service notifications",
        initial=True,
        required=False,
    )

    class Meta:
        model = get_user_model()
        fields = (
            "first_name",
            "last_name",
            "address",
            "postal_code",
            "country",
            "state",
            "city",
            "company_name",
            "company_website",
            "company_vat",
            "news",
        )

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.helper.layout = Layout(
            Fieldset(
                "Billing Details",
                Row(
                    Field("first_name", template="crispy/field.html"),
                    Field("last_name", template="crispy/field.html"),
                    css_class="flex gap-2",
                ),
                "address",
                Row(
                    Field("postal_code", template="crispy/field.html"),
                    Field("city", template="crispy/field.html"),
                    css_class="flex gap-2",
                ),
                Field("country", template="crispy/field.html"),
                "state",
            ),
            Fieldset(
                "Company Details",
                "company_name",
                "company_website",
                "company_vat",
            ),
            Fieldset(
                "Legal",
                Field("news", template="crispy/checkboxfield.html"),
                css_class="text-justify block",
            ),
            ButtonHolder(ButtonFullWidth("submit", "Submit", css_class="mt-4")),
        )


class CustomPasswordChangeForm(TailwindFormMixin, PasswordChangeForm):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.helper.layout = Layout(
            "old_password",
            "new_password1",
            "new_password2",
            ButtonHolder(ButtonFullWidth("submit", "Submit", css_class="mt-4")),
        )



from django.contrib import auth, messages
from django.contrib.auth import update_session_auth_hash
from django.contrib.auth import views as auth_views
from django.contrib.auth.decorators import login_required
from django.contrib.auth.forms import AuthenticationForm
from django.shortcuts import redirect, render
from django.urls import reverse_lazy

from tuhls.core.utils import AuthenticatedHttpRequest, ip_to_country, request_to_ip

from . import forms


def login(request):
    form = AuthenticationForm(data=request.POST)
    if request.method == "POST":
        if form.is_valid() and form.clean():
            auth.login(request, form.get_user())
            return redirect("dashboard")

        messages.error(request, ["Login failed!", "Username or password not correct"])

    return render(request, "auth/login.html", {"form": form})


class PasswordResetView(auth_views.PasswordResetView):
    form_class = forms.PasswordResetForm
    template_name = "password_reset_form.html"
    email_template_name = "password_reset_email.html"


class PasswordResetDoneView(auth_views.PasswordResetDoneView):
    template_name = "password_reset_done.html"


class PasswordResetConfirmView(auth_views.PasswordResetConfirmView):
    template_name = "password_reset_confirm.html"
    post_reset_login = True
    form_class = forms.SetPasswordForm
    success_url = reverse_lazy("dashboard")


def signup(request):
    country_code = ip_to_country(request_to_ip(request))

    if request.method == "POST":
        form = forms.SignupForm(request.POST)
        if form.is_valid():
            form.save()
            auth.login(
                request,
                auth.authenticate(
                    email=form.cleaned_data.get("email"),
                    password=form.cleaned_data.get("password"),
                ),
            )
            return redirect("dashboard")
        else:
            messages.error(request, ["Signup failed!", "Please correct the errors."])
    else:
        form = forms.SignupForm(initial={"country": country_code})

    return render(request, "auth/signup.html", {"form": form})


@login_required()
def dashboard(request: AuthenticatedHttpRequest):
    return render(
        request,
        "dashboard/dashboard.html",
        {
            "token": request.user.get_token(),
            "user": request.user,
        },
    )


@login_required()
def account(request):
    if request.method == "POST":
        form = forms.AccountForm(request.POST, instance=request.user)
        if form.is_valid():
            form.save()
    else:
        form = forms.AccountForm(instance=request.user)

    return render(request, "dashboard/account.html", {"form": form})


@login_required()
def change_password(request):
    if request.method == "POST":
        form = forms.CustomPasswordChangeForm(request.user, request.POST)
        if form.is_valid():
            user = form.save()
            update_session_auth_hash(request, user)  # Important!
            messages.success(request, "Your password was successfully updated!")
            return redirect("change_password")
        else:
            messages.error(
                request, ["Password change failed!", "Please correct the errors."]
            )
    else:
        form = forms.CustomPasswordChangeForm(request.user)
    return render(request, "dashboard/change_password.html", {"form": form})



import os

import typer

from tuhls.gosu import cmds

app = typer.Typer()
app.add_typer(cmds.ci.app, name="ci")
app.add_typer(cmds.django.app, name="django")


def import_from_path(path, module_name):
    import importlib.util

    spec = importlib.util.spec_from_file_location(module_name, path)
    foo = importlib.util.module_from_spec(spec)
    spec.loader.exec_module(foo)
    return foo


def find_local_gosu_cmds():
    cmds_path = os.getcwd() + "/cmds.py"
    if not os.path.isfile(cmds_path):
        return

    import_from_path(cmds_path, "gosu.user.cmds")


if __name__ == "gosu.main":
    find_local_gosu_cmds()



import glob
import os

import typer
from plumbum import FG, local

from tuhls.gosu import _get_dot_env, _get_pyproject_toml

app = typer.Typer()


def _manage_env_vars():
    env_vars = _get_dot_env()
    for name, value in env_vars.items():
        local.env[name] = value

    return env_vars


def _run(*cmds):
    _manage_env_vars()
    print(cmds)  # noqa
    local[cmds[0]][cmds[1:]] & FG


def _get_django_app_src():
    try:
        return _get_pyproject_toml()["gosu"]["django_app_src"]
    except Exception:
        return "example"


def _pm(*cmds):
    local.cwd.chdir(_get_django_app_src())
    _run("python", "manage.py", *cmds)
    local.cwd.chdir("..")


@app.command(
    context_settings={"allow_extra_args": True, "ignore_unknown_options": True}
)
def pm(ctx: typer.Context):
    _pm(*ctx.args)


@app.command()
def migrate():
    _pm("makemigrations")
    _pm("migrate")


@app.command()
def gunicorn():
    _pm("collectstatic", "--no-input")
    _pm("migrate")
    local.cwd.chdir(_get_django_app_src())
    _run(
        "gunicorn",
        "--bind",
        "0.0.0.0:8000",
        "--workers=2",
        "--worker-tmp-dir",
        "/dev/shm",
        "base.wsgi:application",
    )


@app.command()
def test():
    _manage_env_vars()
    local.env["DEFAULT_CACHE"] = "locmemcache://"
    local.env["QUEUE_CACHE"] = "locmemcache://"
    rcfile = f"--rcfile={os.path.dirname(__file__)}/../.coveragerc"
    _run(
        "coverage",
        "run",
        "--concurrency=multiprocessing",
        "--parallel-mode",
        rcfile,
        f"{_get_django_app_src()}/manage.py",
        "test",
        "--parallel=3",
        ".",
    )
    _run("coverage", "combine", rcfile)
    _run("coverage", "report", "-i", rcfile)


@app.command()
def precommit():
    fix()
    test()
    build()


@app.command()
def notebook():
    local.env["DJANGO_ALLOW_ASYNC_UNSAFE"] = True
    _pm("shell_plus", "--notebook")


@app.command(
    context_settings={"allow_extra_args": True, "ignore_unknown_options": True}
)
def run(ctx: typer.Context):
    _run(*ctx.args)


def _get_pkg_pyfiles():
    files = list(
        filter(
            lambda e: all(
                [not (x in e) for x in ["node_modules", "migrations", "build"]]
            ),
            glob.glob("**/*.py", recursive=True),
        )
    )
    print(files)  # noqa
    return files


@app.command()
def fix():
    _run("pyupgrade", "--py39-plus", "--exit-zero-even-if-changed", *_get_pkg_pyfiles())
    _run("isort", "--profile", "black", *_get_pkg_pyfiles())
    _run("black", *_get_pkg_pyfiles())
    _run("flake8", "--config", f"{os.path.dirname(__file__)}/../.flake8")


@app.command()
def lint():
    _run("pyupgrade", "--py39-plus", *_get_pkg_pyfiles())
    _run("isort", "--profile", "black", "-c", *_get_pkg_pyfiles())
    _run("black", "--check", *_get_pkg_pyfiles())
    _run("flake8", "--config", f"{os.path.dirname(__file__)}/../.flake8")


@app.command()
def build():
    _run("flit", "build")



import os
import re
import subprocess
from enum import Enum

import typer
from plumbum import FG, ProcessExecutionError
from plumbum import local as lr

from tuhls.gosu import _get_pyproject_toml

app = typer.Typer()


def git(*args, output=True):
    r = subprocess.check_output(["git"] + list(args))
    if output:
        print(args)  # noqa
        print(r)  # noqa
    return r


def git_num_changes():
    return len(git("status", "--porcelain").splitlines())


def _get_version():
    return git("describe", "--tags").decode().strip()


@app.command()
def push_repo():
    git(
        "remote",
        "set-url",
        "--push",
        "origin",
        re.sub(r".+@([^/]+)/", r"git@\1:", os.environ["CI_REPOSITORY_URL"]),
    )
    git("push", "-o", "ci.skip", "origin", get_version())


class SemverPart(str, Enum):
    patch = "patch"
    minor = "minor"
    major = "major"


@app.command()
def bump_version(semver_part: SemverPart = SemverPart.patch):
    import semver
    import toml

    git("fetch", "--tags", "-f")
    git("tag")
    try:
        v = _get_version()
        if semver_part.value == SemverPart.patch:
            n = semver.bump_patch(v)
        elif semver_part.value == SemverPart.minor:
            n = semver.bump_minor(v)
        elif semver_part.value == SemverPart.major:
            n = semver.bump_major(v)
    except (subprocess.CalledProcessError, ValueError):
        print("initialise versioning with 1.0.0")  # noqa
        git("tag", "1.0.0")
        return

    if "-" not in v:
        return

    print(f"bump from {v} to {n}")  # noqa
    data = _get_pyproject_toml()
    data["project"]["version"] = n
    with open("pyproject.toml", "w") as f:
        toml.dump(data, f)
    git("add", ".")
    git("commit", "--allow-empty", "-m", f"bump version to {n}")
    git("push", "-o", "ci.skip")
    git("tag", "-a", n, "-m", f"release {n}")
    git("push", "--follow-tags")


@app.command()
def get_version():
    print(f"current version: {_get_version()}")  # noqa


@app.command()
def local():
    has_changes = git_num_changes() > 0
    try:
        if has_changes:
            git("add", ".")
            git("commit", "-m", "local debug commit")
        (
            lr["gitlab-ci-local"][
                "--privileged",
            ]
            & FG
        )
    except ProcessExecutionError:
        exit(1)
    finally:
        if has_changes:
            git("reset", "HEAD~1")


def get_project_name():
    return _get_pyproject_toml()["project"]["name"]


def get_project_version():
    return _get_pyproject_toml()["project"]["version"]


def get_name_version():
    return f"{get_project_name()}:{get_project_version()}"


@app.command()
def docker():
    lr["cp"]["Dockerfile", "dist/Dockerfile"] & FG
    lr.cwd.chdir("dist")
    (
        lr["docker"][
            "build",
            "--no-cache",
            "-t",
            get_project_name(),
            "-t",
            get_name_version(),
            ".",
        ]
        & FG
    )


@app.command()
def podman_login():
    (
        lr["podman"][
            "login",
            "-u",
            os.environ["CI_REGISTRY_USER"],
            "-p",
            os.environ["CI_REGISTRY_PASSWORD"],
            os.environ["CI_REGISTRY"],
        ]
        & FG
    )


@app.command()
def podman_build():
    lr["cp"]["Dockerfile", "dist/Dockerfile"] & FG
    lr.cwd.chdir("dist")
    registry_name = os.environ["CI_REGISTRY_IMAGE"]
    (
        lr["podman"][
            "build",
            "-t",
            f"{registry_name}/{get_project_name()}",
            "-t",
            f"{registry_name}/{get_name_version()}",
            ".",
        ]
        & FG
    )


@app.command()
def podman_push():
    registry_name = os.environ["CI_REGISTRY_IMAGE"]
    lr["podman"]["push", f"{registry_name}/{get_project_name()}"] & FG
    lr["podman"]["push", f"{registry_name}/{get_name_version()}"] & FG


@app.command()
def twine_gitlab_upload():
    if "CI_COMMIT_TAG" not in lr.env:
        return
    lr.env["TWINE_PASSWORD"] = lr.env["CI_JOB_TOKEN"]
    lr.env["TWINE_USERNAME"] = "gitlab-ci-token"
    (
        lr["python"][
            "-m",
            "twine",
            "upload",
            "--repository-url",
            f'{lr.env["CI_API_V4_URL"]}/projects/{lr.env["CI_PROJECT_ID"]}/packages/pypi',
            "dist/*",
        ]
        & FG
    )


@app.command()
def twine_pypi_upload():
    if "CI_COMMIT_TAG" not in lr.env:
        return
    lr.env["TWINE_PASSWORD"] = lr.env["PYPI_TOKEN"]
    lr.env["TWINE_USERNAME"] = "__token__"
    (
        lr["python"][
            "-m",
            "twine",
            "upload",
            "--repository-url",
            "https://upload.pypi.org/legacy/",
            "dist/*",
        ]
        & FG
    )



import os
import os.path


def _find_recursive(name):
    parts = os.getcwd().split(os.sep)
    for x in range(len(parts), 0, -1):
        f = os.sep.join(parts[0:x] + [name])
        if os.path.isfile(f):
            return f
    raise FileNotFoundError


def _get_dot_env():
    from dotenv import dotenv_values

    try:
        return dotenv_values(_find_recursive(".env"))
    except FileNotFoundError:
        return {}


def _get_pyproject_toml():
    import toml

    return toml.load(_find_recursive("pyproject.toml"))



import os
import sys


def build_env():
    import configurations

    configurations.setup()


def run():
    from configurations.management import execute_from_command_line

    build_env()
    execute_from_command_line(sys.argv)


def run_example():
    from django.core.management import execute_from_command_line

    os.environ.setdefault("DJANGO_SETTINGS_MODULE", "example.settings")
    execute_from_command_line(sys.argv)


def wsgi():
    from configurations.wsgi import get_wsgi_application

    build_env()
    return get_wsgi_application()



from sortedcontainers import SortedDict

menu = {
    "main": SortedDict(
        {
            100: SortedDict(
                {
                    100: ["dashboard", "Dashboard", "user-circle"],
                }
            ),
            200: SortedDict(
                {
                    100: ["account", "Account", "user-group"],
                    200: ["change_password", "Change Password", "key"],
                }
            ),
            999: SortedDict(
                {
                    100: ["sign_out", "Sign Out", "logout"],
                }
            ),
        }
    ),
}
